package com.emperador.emperador_player

import android.app.PendingIntent
import android.content.Context
import android.graphics.Bitmap
import android.util.Log
import androidx.annotation.Nullable
import com.google.android.exoplayer2.*
import com.google.android.exoplayer2.metadata.MetadataOutput
import com.google.android.exoplayer2.metadata.icy.IcyHeaders
import com.google.android.exoplayer2.metadata.icy.IcyInfo
import com.google.android.exoplayer2.source.DefaultMediaSourceFactory
import com.google.android.exoplayer2.ui.PlayerNotificationManager
import com.google.android.exoplayer2.ui.PlayerNotificationManager.BitmapCallback
import com.google.android.exoplayer2.ui.PlayerNotificationManager.MediaDescriptionAdapter
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodChannel


class AudioPlayer(var context: Context, binaryMessenger: BinaryMessenger) {
    private lateinit var playerNotificationManager: PlayerNotificationManager
    var methodChannel = MethodChannel(binaryMessenger, "com.emperador.player/channel")
    lateinit var player: ExoPlayer
    var isConfigured = false


    fun config(url: String?, metaDivider: String?) {
        methodChannel.invokeMethod("idle", null)
        val adapter = DescriptionAdapter()

        playerNotificationManager = PlayerNotificationManager.Builder(
            context, 1768945, "CHANNEL_ID"
        ).build()

        if (url == null) return

        player = ExoPlayer.Builder(context)
            .setMediaSourceFactory(DefaultMediaSourceFactory(context).setLiveTargetOffsetMs(5000))
            .build()

        val mediaItem: MediaItem = MediaItem.Builder()
            .setUri(url)
//            .setLiveMaxPlaybackSpeed(1.02f)
            .build()


        val eventListener = object : Player.Listener {
            override fun onPlaybackStateChanged(playbackState: Int) {
                if (playbackState == Player.STATE_IDLE) methodChannel.invokeMethod("idle", null)
                if (playbackState == Player.STATE_BUFFERING) methodChannel.invokeMethod("loading", null)
                if (playbackState == Player.STATE_READY) methodChannel.invokeMethod("ready", null)
                if (playbackState == Player.STATE_ENDED) methodChannel.invokeMethod("idle", null)
                if (isPlaying()) methodChannel.invokeMethod("playing", null)
            }

            override fun onPlayerError(error: PlaybackException) {
                methodChannel.invokeMethod("error", error)
            }

            override fun onMediaMetadataChanged(mediaMetadata: MediaMetadata) {
                Log.e("TAG", mediaMetadata.albumArtist.toString())
                Log.e("TAG", mediaMetadata.albumTitle.toString())
                Log.e("TAG", mediaMetadata.title.toString())
                Log.e("TAG", mediaMetadata.subtitle.toString())

                val entry = mediaMetadata.title.toString()
                var divider = "-"
                var artist = ""
                var song = ""
                val genre = ""
                if (metaDivider != null) divider = metaDivider
                val splitted = entry.split(divider)
                if (splitted.isNotEmpty())
                    artist = splitted[0].trim()
                if (splitted.size > 1)
                    song = splitted[1].trim()
                methodChannel.invokeMethod("metadata", "{\"artist\":\"$artist\",\"song\":\"$song\",\"genre\":\"$genre\"}")
                super.onMediaMetadataChanged(mediaMetadata)
            }
        }

        playerNotificationManager.setPlayer(player)

        player.addListener(eventListener)
        player.setMediaItem(mediaItem)
        player.prepare()

        methodChannel.invokeMethod("ready", null)
        isConfigured = true

    }


    fun syncState() {
        if (isConfigured) methodChannel.invokeMethod("ready", null)
        if (isPlaying()) methodChannel.invokeMethod("playing", null)
    }

    fun isPlaying(): Boolean {
        return player.playbackState == Player.STATE_READY && player.playWhenReady
    }

    fun play() {
        player.play()
    }

    fun close() {
        playerNotificationManager.setPlayer(null)
        player.release()
    }

    fun stop() {
        player.stop()
    }

    fun pause() {
        player.pause()
    }


}

private class DescriptionAdapter : MediaDescriptionAdapter {
    override fun getCurrentContentTitle(player: Player): String {
        return "title"
    }


    @Nullable
    override fun getCurrentContentText(player: Player): String {
        return " description"
    }


    @Nullable
    override fun getCurrentLargeIcon(player: Player, callback: BitmapCallback): Bitmap? {
        return null
    }


    @Nullable
    override fun createCurrentContentIntent(player: Player): PendingIntent? {
        return null
    }

}

